@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -75,17 +75,18 @@ void flag_heads_kernel(Type* device_input, long long* device_heads)
     Type input[ItemsPerThread];
     rocprim::block_load_direct_blocked(lid, device_input + block_offset, input);
 
-    rocprim::block_discontinuity<Type, BlockSize> bdiscontinuity;
+    rocprim::block_discontinuity<Type, BlockSize>              bdiscontinuity;
+    __shared__ typename decltype(bdiscontinuity)::storage_type storage;
 
     FlagType head_flags[ItemsPerThread];
     if(blockIdx.x % 2 == 1)
     {
         const Type tile_predecessor_item = device_input[block_offset - 1];
-        bdiscontinuity.flag_heads(head_flags, tile_predecessor_item, input, FlagOpType());
+        bdiscontinuity.flag_heads(head_flags, tile_predecessor_item, input, FlagOpType(), storage);
     }
     else
     {
-        bdiscontinuity.flag_heads(head_flags, input, FlagOpType());
+        bdiscontinuity.flag_heads(head_flags, input, FlagOpType(), storage);
     }
 
     rocprim::block_store_direct_blocked(lid, device_heads + block_offset, head_flags);
@@ -109,17 +110,18 @@ void flag_tails_kernel(Type* device_input, long long* device_tails)
     Type input[ItemsPerThread];
     rocprim::block_load_direct_blocked(lid, device_input + block_offset, input);
 
-    rocprim::block_discontinuity<Type, BlockSize> bdiscontinuity;
+    rocprim::block_discontinuity<Type, BlockSize>              bdiscontinuity;
+    __shared__ typename decltype(bdiscontinuity)::storage_type storage;
 
     FlagType tail_flags[ItemsPerThread];
     if(blockIdx.x % 2 == 0)
     {
         const Type tile_successor_item = device_input[block_offset + items_per_block];
-        bdiscontinuity.flag_tails(tail_flags, tile_successor_item, input, FlagOpType());
+        bdiscontinuity.flag_tails(tail_flags, tile_successor_item, input, FlagOpType(), storage);
     }
     else
     {
-        bdiscontinuity.flag_tails(tail_flags, input, FlagOpType());
+        bdiscontinuity.flag_tails(tail_flags, input, FlagOpType(), storage);
     }
 
     rocprim::block_store_direct_blocked(lid, device_tails + block_offset, tail_flags);
@@ -143,29 +145,46 @@ void flag_heads_and_tails_kernel(Type* device_input, long long* device_heads, lo
     Type input[ItemsPerThread];
     rocprim::block_load_direct_blocked(lid, device_input + block_offset, input);
 
-    rocprim::block_discontinuity<Type, BlockSize> bdiscontinuity;
+    rocprim::block_discontinuity<Type, BlockSize>              bdiscontinuity;
+    __shared__ typename decltype(bdiscontinuity)::storage_type storage;
 
     FlagType head_flags[ItemsPerThread];
     FlagType tail_flags[ItemsPerThread];
     if(blockIdx.x % 4 == 0)
     {
         const Type tile_successor_item = device_input[block_offset + items_per_block];
-        bdiscontinuity.flag_heads_and_tails(head_flags, tail_flags, tile_successor_item, input, FlagOpType());
+        bdiscontinuity.flag_heads_and_tails(head_flags,
+                                            tail_flags,
+                                            tile_successor_item,
+                                            input,
+                                            FlagOpType(),
+                                            storage);
     }
     else if(blockIdx.x % 4 == 1)
     {
         const Type tile_predecessor_item = device_input[block_offset - 1];
-        const Type tile_successor_item = device_input[block_offset + items_per_block];
-        bdiscontinuity.flag_heads_and_tails(head_flags, tile_predecessor_item, tail_flags, tile_successor_item, input, FlagOpType());
+        const Type tile_successor_item   = device_input[block_offset + items_per_block];
+        bdiscontinuity.flag_heads_and_tails(head_flags,
+                                            tile_predecessor_item,
+                                            tail_flags,
+                                            tile_successor_item,
+                                            input,
+                                            FlagOpType(),
+                                            storage);
     }
     else if(blockIdx.x % 4 == 2)
     {
         const Type tile_predecessor_item = device_input[block_offset - 1];
-        bdiscontinuity.flag_heads_and_tails(head_flags, tile_predecessor_item, tail_flags, input, FlagOpType());
+        bdiscontinuity.flag_heads_and_tails(head_flags,
+                                            tile_predecessor_item,
+                                            tail_flags,
+                                            input,
+                                            FlagOpType(),
+                                            storage);
     }
     else if(blockIdx.x % 4 == 3)
     {
-        bdiscontinuity.flag_heads_and_tails(head_flags, tail_flags, input, FlagOpType());
+        bdiscontinuity.flag_heads_and_tails(head_flags, tail_flags, input, FlagOpType(), storage);
     }
 
     rocprim::block_store_direct_blocked(lid, device_heads + block_offset, head_flags);
@@ -185,11 +204,15 @@ auto test_block_discontinuity()
 {
     using type = Type;
     // std::vector<bool> is a special case that will cause an error in hipMemcpy
+    // rocprim::half/rocprim::bfloat16 are special cases that cannot be compared '=='
+    // in ASSERT_EQ
     using stored_flag_type = typename std::conditional<
-                               std::is_same<bool, FlagType>::value,
-                               int,
-                               FlagType
-                           >::type;
+        std::is_same<bool, FlagType>::value,
+        int,
+        typename std::conditional<std::is_same<rocprim::half, FlagType>::value
+                                      || std::is_same<rocprim::bfloat16, FlagType>::value,
+                                  float,
+                                  FlagType>::type>::type;
     using flag_type = FlagType;
     using flag_op_type = FlagOpType;
     static constexpr size_t block_size = BlockSize;
@@ -198,6 +221,10 @@ auto test_block_discontinuity()
     const size_t size = items_per_block * 20;
     static constexpr size_t grid_size = size / items_per_block;
 
+    SCOPED_TRACE(testing::Message() << "items_per_block = " << items_per_block);
+    SCOPED_TRACE(testing::Message() << "size = " << size);
+    SCOPED_TRACE(testing::Message() << "grid_size = " << grid_size);
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -223,9 +250,8 @@ auto test_block_discontinuity()
                 const size_t i = bi * items_per_block + ii;
                 if(ii == 0)
                 {
-                    expected_heads[i] = bi % 2 == 1
-                        ? apply(flag_op, input[i - 1], input[i], ii)
-                        : flag_type(true);
+                    expected_heads[i] = bi % 2 == 1 ? apply(flag_op, input[i - 1], input[i], ii)
+                                                    : stored_flag_type(true);
                 }
                 else
                 {
@@ -296,11 +322,15 @@ auto test_block_discontinuity()
 {
     using type = Type;
     // std::vector<bool> is a special case that will cause an error in hipMemcpy
+    // rocprim::half/rocprim::bfloat16 are special cases that cannot be compared '=='
+    // in ASSERT_EQ
     using stored_flag_type = typename std::conditional<
-                               std::is_same<bool, FlagType>::value,
-                               int,
-                               FlagType
-                           >::type;
+        std::is_same<bool, FlagType>::value,
+        int,
+        typename std::conditional<std::is_same<rocprim::half, FlagType>::value
+                                      || std::is_same<rocprim::bfloat16, FlagType>::value,
+                                  float,
+                                  FlagType>::type>::type;
     using flag_type = FlagType;
     using flag_op_type = FlagOpType;
     static constexpr size_t block_size = BlockSize;
@@ -309,6 +339,10 @@ auto test_block_discontinuity()
     const size_t size = items_per_block * 20;
     static constexpr size_t grid_size = size / items_per_block;
 
+    SCOPED_TRACE(testing::Message() << "items_per_block = " << items_per_block);
+    SCOPED_TRACE(testing::Message() << "size = " << size);
+    SCOPED_TRACE(testing::Message() << "grid_size = " << grid_size);
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -334,9 +368,8 @@ auto test_block_discontinuity()
                 const size_t i = bi * items_per_block + ii;
                 if(ii == items_per_block - 1)
                 {
-                    expected_tails[i] = bi % 2 == 0
-                        ? apply(flag_op, input[i], input[i + 1], ii + 1)
-                        : flag_type(true);
+                    expected_tails[i] = bi % 2 == 0 ? apply(flag_op, input[i], input[i + 1], ii + 1)
+                                                    : stored_flag_type(true);
                 }
                 else
                 {
@@ -407,11 +440,15 @@ auto test_block_discontinuity()
 {
     using type = Type;
     // std::vector<bool> is a special case that will cause an error in hipMemcpy
+    // rocprim::half/rocprim::bfloat16 are special cases that cannot be compared '=='
+    // in ASSERT_EQ
     using stored_flag_type = typename std::conditional<
-                               std::is_same<bool, FlagType>::value,
-                               int,
-                               FlagType
-                           >::type;
+        std::is_same<bool, FlagType>::value,
+        int,
+        typename std::conditional<std::is_same<rocprim::half, FlagType>::value
+                                      || std::is_same<rocprim::bfloat16, FlagType>::value,
+                                  float,
+                                  FlagType>::type>::type;
     using flag_type = FlagType;
     using flag_op_type = FlagOpType;
     static constexpr size_t block_size = BlockSize;
@@ -420,6 +457,10 @@ auto test_block_discontinuity()
     const size_t size = items_per_block * 20;
     static constexpr size_t grid_size = size / items_per_block;
 
+    SCOPED_TRACE(testing::Message() << "items_per_block = " << items_per_block);
+    SCOPED_TRACE(testing::Message() << "size = " << size);
+    SCOPED_TRACE(testing::Message() << "grid_size = " << grid_size);
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -448,8 +489,8 @@ auto test_block_discontinuity()
                 if(ii == 0)
                 {
                     expected_heads[i] = (bi % 4 == 1 || bi % 4 == 2)
-                        ? apply(flag_op, input[i - 1], input[i], ii)
-                        : flag_type(true);
+                                            ? apply(flag_op, input[i - 1], input[i], ii)
+                                            : stored_flag_type(true);
                 }
                 else
                 {
@@ -458,8 +499,8 @@ auto test_block_discontinuity()
                 if(ii == items_per_block - 1)
                 {
                     expected_tails[i] = (bi % 4 == 0 || bi % 4 == 1)
-                        ? apply(flag_op, input[i], input[i + 1], ii + 1)
-                        : flag_type(true);
+                                            ? apply(flag_op, input[i], input[i + 1], ii + 1)
+                                            : stored_flag_type(true);
                 }
                 else
                 {
@@ -543,11 +584,14 @@ struct static_for
 {
     static void run()
     {
-        int device_id = test_common_utils::obtain_device_from_ctest();
-        SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
-        HIP_CHECK(hipSetDevice(device_id));
+        {
+            SCOPED_TRACE(testing::Message() << "TestID = " << First);
+            int device_id = test_common_utils::obtain_device_from_ctest();
+            SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
+            HIP_CHECK(hipSetDevice(device_id));
 
-        test_block_discontinuity<Type, FlagType, FlagOpType, Method, BlockSize, items[First]>();
+            test_block_discontinuity<Type, FlagType, FlagOpType, Method, BlockSize, items[First]>();
+        }
         static_for<First + 1, Last, Type, FlagType, FlagOpType, Method, BlockSize>::run();
     }
 };
